# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# This file is in the public domain
### END LICENSE

import gettext
from gettext import gettext as _
gettext.textdomain('qreator')

import gi
gi.require_version('Gdk', '3.0')
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk, Gdk # pylint: disable=E0611
import cairo
import logging
logger = logging.getLogger('qreator')

from qreator_lib import Window
from qreator.AboutQreatorDialog import AboutQreatorDialog
from qreator.PreferencesQreatorDialog import PreferencesQreatorDialog

import qrencode
import array
import Image
import ImageOps

COL_TEXT = 0
COL_PIXBUF = 1
COL_ID = 2

PAGE_HOME = 0
PAGE_HISTORY = 1
PAGE_SETTINGS = 2
PAGE_ABOUT = 3
PAGE_URL = 4


class BarcodeSize(object):
    SMALL = 120
    MEDIUM = 230
    LARGE = 350


# See qreator_lib.Window.py for more details about how this class works
class QreatorWindow(Window):
    __gtype_name__ = "QreatorWindow"

    def finish_initializing(self, builder): # pylint: disable=E1002
        """Set up the main window"""
        super(QreatorWindow, self).finish_initializing(builder)

        self.AboutDialog = AboutQreatorDialog
        self.PreferencesDialog = PreferencesQreatorDialog

        # Code for other initialization actions should be added here.
        self.clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
        self.barcode_size = BarcodeSize.SMALL
        self.pixel_size = 4
        self.surface = None
        self.ui.iconview1.set_text_column(COL_TEXT)
        self.ui.iconview1.set_pixbuf_column(COL_PIXBUF)

        self.ui.notebook1.set_show_tabs(False)
        self.ui.entry1.connect('changed', self.on_entry1_changed)

    def on_toolbuttonHome_clicked(self, widget, data=None):
        self.ui.notebook1.set_current_page(PAGE_HOME)

    def on_toolbuttonHistory_clicked(self, widget, data=None):
        self.ui.notebook1.set_current_page(PAGE_HISTORY)

    def on_toolbuttonSettings_clicked(self, widget, data=None):
        self.ui.notebook1.set_current_page(PAGE_SETTINGS)

    def on_toolbuttonAbout_clicked(self, widget, data=None):
        self.ui.notebook1.set_current_page(PAGE_ABOUT)

    def on_iconview1_item_activated(self, widget, item):
        self.ui.notebook1.set_current_page(PAGE_URL)
        #model = widget.get_model()
        #id = model[item][COL_ID]

    def get_pixbuf_from_drawing_area(self):
        window = self.ui.drawingarea1.get_window()

        src_x, src_y = self.get_centered_coordinates(self.ui.drawingarea1,
                                                     self.surface)
        image_height = self.surface.get_height()
        image_width = self.surface.get_width()

        return Gdk.pixbuf_get_from_window(window, src_x, src_y,
                                          image_width, image_height)

    def on_toolbuttonSave_clicked(self, widget, data=None):
        if not self.surface:
            return

        dialog = Gtk.FileChooserDialog(_("Please choose a file"), self,
            Gtk.FileChooserAction.SAVE,
            (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
             Gtk.STOCK_SAVE, Gtk.ResponseType.OK))

        filter_png = Gtk.FileFilter()
        filter_png.set_name(_("PNG images"))
        filter_png.add_mime_type("image/png")
        dialog.add_filter(filter_png)

        response = dialog.run()

        if response == Gtk.ResponseType.OK:
            # We cannot write directly from the surface, as the
            # Surface.write_to_png() method writes the image in the original
            # size returned by qrencode (i.e. non-scaled), and the
            # SurfacePattern does not have any methods to write to disk.
            # So we read the contents from the Gtk.DrawingArea, put them into
            # a Gdk.Pixbuf and use its 'savev' method to write to disk.

            pixbuf = self.get_pixbuf_from_drawing_area()

            pixbuf.savev(dialog.get_filename(), 'png', [], [])

        dialog.destroy()

    def on_toolbuttonCopy_clicked(self, widget, data=None):
        if not self.surface:
            return

        pixbuf = self.get_pixbuf_from_drawing_area()
        self.clipboard.set_image(pixbuf)

    def on_entry1_changed(self, widget, data=None):
        self.ui.drawingarea1.queue_draw()

    def get_centered_coordinates(self, drawing_area, surface):

        drawing_area_height = drawing_area.get_allocated_height()
        drawing_area_width = drawing_area.get_allocated_width()
        image_height = surface.get_height()
        image_width = surface.get_width()

        return (drawing_area_width / 2 - image_width / 2,
                drawing_area_height / 2 - image_height / 2)

    def set_pixel_size(self, barcode_orig_size):
    
        pixel_size = 1
        
        while pixel_size * barcode_orig_size < self.barcode_size:
            pixel_size += 1

        while ((pixel_size * barcode_orig_size > self.barcode_size) or \
                ((pixel_size * barcode_orig_size) % 2 != 0)):
            pixel_size -= 1
        
        if pixel_size < 0:
            pixel_size = 1
            
        self.pixel_size = pixel_size

    def on_drawingarea1_draw(self, widget, ctx, data=None):
        text = self.ui.entry1.get_text()
        if text == '':
            return

        # Create the QR code
        version, size, im = qrencode.encode(text)

        # Resize the original image
        self.set_pixel_size(size)
        print self.pixel_size, size
        im = im.resize((self.pixel_size * size, self.pixel_size * size),
                       Image.NEAREST)
        im = im.convert('RGBA')

        # Add a border
        im = ImageOps.expand(im, border=(self.barcode_size - im.size[0]) / 2,
                             fill='white')
        
        # Convert the Python PIL image to a Cairo surface.
        # Notice the conversion to BGRA that Cairo expects
        # See http://cairographics.org/pythoncairopil/ and
        # http://mail.gnome.org/archives/gtkmm-list/2007-May/msg00111.html
        bytearr = array.array('B', im.tostring("raw", "BGRA", 0, 1))
        height, width = im.size
        print im.size

        self.surface = cairo.ImageSurface.create_for_data(bytearr,
                                                     cairo.FORMAT_ARGB32,
                                                     width, height,
                                                     width * 4)

        # Center the image in the drawing area
        centered_width, centered_height = \
            self.get_centered_coordinates(widget, self.surface)
        ctx.translate(centered_width, centered_height)
        
        # Set the surface as the context's source
        ctx.set_source_surface(self.surface)

        # Render the image
        ctx.paint()
